package com.company.anonym.screen.main;

import io.jmix.core.FileRef;
import io.jmix.ui.Dialogs;
import io.jmix.ui.Notifications;
import io.jmix.ui.ScreenTools;
import io.jmix.ui.action.DialogAction;
import io.jmix.ui.component.*;
import io.jmix.ui.component.mainwindow.Drawer;
import io.jmix.ui.icon.JmixIcon;
import io.jmix.ui.navigation.Route;
import io.jmix.ui.screen.Screen;
import io.jmix.ui.screen.Subscribe;
import io.jmix.ui.screen.UiController;
import io.jmix.ui.screen.UiControllerUtils;
import io.jmix.ui.screen.UiDescriptor;
import io.jmix.ui.upload.TemporaryStorage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.File;

@UiController("MainScreen")
@UiDescriptor("main-screen.xml")
@Route(path = "main", root = true)
public class MainScreen extends Screen implements Window.HasWorkArea {
    private final Logger log = LoggerFactory.getLogger(MainScreen.class);

    @Autowired
    private ScreenTools screenTools;
    @Autowired
    private AppWorkArea workArea;
    @Autowired
    private Drawer drawer;
    @Autowired
    private Button collapseDrawerButton;
    @Autowired
    private Notifications notifications;
    @Autowired
    private Dialogs dialogs;
    @Autowired
    private FileStorageUploadField myFileStorageUpload;
    @Autowired
    private TemporaryStorage temporaryStorage;

    @Override
    public AppWorkArea getWorkArea() {
        return workArea;
    }

    @Subscribe("collapseDrawerButton")
    private void onCollapseDrawerButtonClick(Button.ClickEvent event) {
        drawer.toggle();
        if (drawer.isCollapsed()) {
            collapseDrawerButton.setIconFromSet(JmixIcon.CHEVRON_RIGHT);
        } else {
            collapseDrawerButton.setIconFromSet(JmixIcon.CHEVRON_LEFT);
        }
    }

    @Subscribe
    public void onAfterShow(AfterShowEvent event) {
        screenTools.openDefaultScreen(
                UiControllerUtils.getScreenContext(this).getScreens());

        screenTools.handleRedirect();
    }

    @Subscribe("myFileStorageUpload")
    public void onMyFileStorageUploadFileUploadSucceed(SingleFileUploadField.FileUploadSucceedEvent event) {
        File file = temporaryStorage.getFile(myFileStorageUpload.getFileId());
        if (file != null) {
            notifications.create(Notifications.NotificationType.TRAY)
                    .withCaption("onMyFileStorageUploadFileUploadSucceed - File Stored Successfully: " + file.getAbsolutePath())
                    .show();
        } else {
            String failedMessage = "onMyFileStorageUploadFileUploadSucceed - File Store Failed: " + event.getFileName();
            notifications.create(Notifications.NotificationType.ERROR)
                    .withCaption(failedMessage)
                    .show();
            log.error(failedMessage);
            return;
        }
    }

    @Subscribe("myFileStorageUpload")
    public void onMyFileStorageUploadBeforeValueClear(SingleFileUploadField.BeforeValueClearEvent event) {
        event.preventClearAction();
    }

    @Subscribe("myFileStorageUpload")
    public void onMyFileStorageUploadAfterValueClear(SingleFileUploadField.AfterValueClearEvent event) {
        dialogs.createOptionDialog()
                .withCaption("Confirmation")
                .withMessage("Clear myFileStorageUpload?")
                .withActions(
                        new DialogAction(DialogAction.Type.YES)
                                .withHandler(e -> {
                                    myFileStorageUpload.clear();
                                    myFileStorageUpload.setShowClearButton(false);
                                }),
                        new DialogAction(DialogAction.Type.NO)
                )
                .show();
    }

    @Subscribe("myFileStorageUpload")
    public void onMyFileStorageUploadValueChange(HasValue.ValueChangeEvent<FileRef> event) {
        notifications.create(Notifications.NotificationType.HUMANIZED)
                .withCaption("onMyFileStorageUploadValueChange - value was changed")
                .show();
    }
}
